@@ -23,8 +23,12 @@ from functools import wraps
 import memcache
 try:
     from hashlib import md5
-except:
+except ImportError:
     from md5 import md5
+try:
+    import json
+except ImportError:
+    import simplejson as json
 
 __all__ = ['CloudFilesFS']
 
@@ -241,6 +245,21 @@ class CloudFilesFD(object):
         else:
             raise IOSError(EPERM, "Seek not available for write operations")
 
+class CacheEncoder(json.JSONEncoder):
+    """JSONEncoder to encode the os.stat_result values into a list"""
+    def default(self, obj):
+        if isinstance(obj, os.stat_result):
+            return tuple(obj)
+        return json.JSONEncoder.default(self, obj)
+
+def serialize(obj):
+    '''Serialize a cache dict into a JSON object'''
+    return json.dumps(obj, cls=CacheEncoder)
+
+def unserialize(js):
+    '''Unserialize a JSON object into a cache dict'''
+    return dict(((smart_str(key), os.stat_result(value)) for key, value in json.loads(js).iteritems()))
+
 class ListDirCache(object):
     '''
     Cache for listdir.  This is to cache the very common case when we
@@ -249,6 +268,7 @@ class ListDirCache(object):
     own caching here to avoid the stat calls each making a connection.
     '''
     MAX_CACHE_TIME = 10         # seconds to cache the listdir for
+    MIN_COMPRESS_LEN = 4096     # min length in bytes to compress cache entries
     memcache = None
 
     def __init__(self, cffs):
@@ -272,13 +292,17 @@ class ListDirCache(object):
         '''Flush the listdir cache'''
         logging.debug("cache flush, current path: %s request: %s" % (self.path, path))
         if self.memcache:
-            if self.path is not None:
-                logging.debug("flushing memcache for %r" % self.path)
-                self.memcache.delete(self.key(self.path))
-            if path and self.path != path:
+            if path is not None:
                 logging.debug("flushing memcache for %r" % path)
                 self.memcache.delete(self.key(path))
-        self.cache = None
+                if self.path == path:
+                    self.cache = None
+            elif self.path is not None:
+                logging.debug("flushing memcache for %r" % self.path)
+                self.memcache.delete(self.key(path))
+                self.cache = None
+        else:
+            self.cache = None
 
     def _make_stat(self, last_modified=None, content_type="application/directory", count=1, bytes=0, **kwargs):
         '''Make a stat object from the parameters passed in from'''
@@ -324,7 +348,7 @@ class ListDirCache(object):
             nbobjects = len(newobjects)
             logging.debug("number of objects after marker %s: %s" % (lastobjectname, nbobjects))
             # add the new list to current list
-            objects = objects + newobjects
+            objects.extend(newobjects)
         logging.debug("total number of objects %s:" % len(objects))
 
         for obj in objects:
@@ -370,6 +394,7 @@ class ListDirCache(object):
         if self.memcache:
             cache = self.memcache.get(self.key(path))
             if cache:
+                cache = unserialize(cache)
                 logging.debug("memcache hit %r" % self.key(path))
             else:
                 logging.debug("memcache miss %r" % self.key(path))
@@ -381,8 +406,10 @@ class ListDirCache(object):
                 container, obj = parse_fspath(path)
                 self.listdir_container(cache, container, obj)
             if self.memcache:
-                self.memcache.set(self.key(path), cache, self.MAX_CACHE_TIME)
-                logging.debug("memcache stored %r" % self.key(path))
+                if self.memcache.set(self.key(path), serialize(cache), self.MAX_CACHE_TIME, min_compress_len=self.MIN_COMPRESS_LEN):
+                    logging.debug("memcache stored %r" % self.key(path))
+                else:
+                    logging.warning("Failed to store the cache")
         self.cache = cache
         self.path = path
         self.when = time.time()
@@ -399,16 +426,15 @@ class ListDirCache(object):
 
     def valid(self, path):
         '''Check the cache is valid for the container and directory path'''
-        if self.memcache:
-            cache = self.memcache.get(self.key(path))
-            if cache:
-                logging.debug("memcache hit %r" % self.key(path))
-                self.cache = cache
-                self.path = path
-                return True
-        if not self.cache:
-            return False
-        if self.path != path:
+        if not self.cache or self.path != path:
+            if self.memcache:
+                cache = self.memcache.get(self.key(path))
+                if cache:
+                    cache = unserialize(cache)
+                    logging.debug("memcache hit %r" % self.key(path))
+                    self.cache = cache
+                    self.path = path
+                    return True
             return False
         age = time.time() - self.when
         return age < self.MAX_CACHE_TIME
